// Copyright  1999 Branko Dimitrijevic
// bepp@bepp.8m.com
// http://bepp.8m.com

unit DBFieldComboBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, db;

type

  TFieldKinds = set of TFieldKind;
  // RelateTarget can be nil.
  TRelateEvent = procedure(const Sender, RelateTarget: TComponent; var DefaultRelating: boolean) of object;

  TDBFieldComboBox = class(TComboBox)
  private
    FDataLink: TDataLink;
    FFieldKinds: TFieldKinds;
    FRelatedComponent: TComponent;
    FOnRelate: TRelateEvent;
    function GetDataSource: TDataSource;
    procedure SetDataSource(const Value: TDataSource);
    procedure FillItems;
  protected
    procedure Click; override;
    procedure DropDown; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Relate; dynamic;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property FieldKinds: TFieldKinds read FFieldKinds write FFieldKinds;
    property RelatedComponent: TComponent read FRelatedComponent write FRelatedComponent;
    property OnRelate: TRelateEvent read FOnRelate write FOnRelate;
  end;

  // TODO: Filtering of indexed and non-indexed fields.

  // TODO: Ability to use DisplayLabel instead of FieldName.

procedure Register;



implementation

{$R *.dcr}

uses
  DBLocateEdit, BDB_Shared;

procedure Register;
begin
  RegisterComponents('BDB', [TDBFieldComboBox]);
end;



{ TDBFieldComboBox }



////////////////////////////////////////////////////////////////////////////////
// private

function TDBFieldComboBox.GetDataSource: TDataSource;
begin
  result := FDataLink.DataSource
end;

procedure TDBFieldComboBox.SetDataSource(const Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  FillItems;
  Text := ''
end;

procedure TDBFieldComboBox.FillItems;
var
  i: integer;
begin
  //if Assigned(FDataLink.DataSet) and Assigned(FDataLink.DataSet.Fields) then
  if Assigned(FDataLink.DataSet) and FDataLink.DataSet.Active then // C++Builder compatible.
  begin
    Items.Clear;
    for i := 0 to FDataLink.DataSet.FieldCount-1 do
    begin
      with FDataLink.DataSet.Fields[i] do
      begin
        if FieldKind in FFieldKinds then
          Items.Append(FDataLink.DataSet.Fields[i].FieldName)
      end
    end
  end;
end;



////////////////////////////////////////////////////////////////////////////////
// protected

procedure TDBFieldComboBox.Click;
begin
  inherited;
  Relate
end;

procedure TDBFieldComboBox.DropDown;
begin
  FillItems;
  inherited;
end;

procedure TDBFieldComboBox.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  if (Operation = opRemove) and (AComponent = DataSource) then
    DataSource := nil;
  inherited
end;

procedure TDBFieldComboBox.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;
  if Key = VK_RETURN then
    Relate
end;



////////////////////////////////////////////////////////////////////////////////
// public

constructor TDBFieldComboBox.Create(AOwner: TComponent);
begin
  inherited;
  FDataLink := TDataLink.Create;
  FFieldKinds := [fkData]
end;

destructor TDBFieldComboBox.Destroy;
begin
  FDataLink.Free;
  inherited
end;

// Calls OnRelate and implements automatic updating of related
// component's DataField (only if DataSource matches).
procedure TDBFieldComboBox.Relate;
var
  DefaultRelating: boolean;
begin

  //if not Assigned(FRelatedComponent) then
  //  exit;
  DefaultRelating := true;
  if Assigned(FOnRelate) then
    FOnRelate(self,FRelatedComponent,DefaultRelating);

  if DefaultRelating and Assigned(FRelatedComponent) then
  begin
    if FRelatedComponent is TDBLocateEdit then
      if TDBLocateEdit(FRelatedComponent).DataSource = DataSource then
         TDBLocateEdit(FRelatedComponent).DataField := Text
  end

end;



end.
